from io import BytesIO
import random
from typing import Optional

import chess
import chess.svg
from cairosvg import svg2png

from aiogram.types import InlineKeyboardMarkup, InlineKeyboardButton


def generate_board_image(board: chess.Board) -> bytes:
    svg_image = chess.svg.board(board)
    png_image = BytesIO()
    svg2png(bytestring=svg_image, write_to=png_image)
    png_image.seek(0)
    return png_image.read()


def first_tap_keyboard(board: chess.Board) -> InlineKeyboardMarkup:
    kb_list = []
    squares_added = set()
    for move in board.legal_moves:
        square = move.from_square
        if square in squares_added:
            continue
        squares_added.add(square)

        kb_list.append([
            InlineKeyboardButton(
                text=f"{board.piece_at(square).unicode_symbol()} {chess.square_name(square)}",
                callback_data="turn_" + chess.square_name(square),
            )
        ])

    keyboard = InlineKeyboardMarkup(inline_keyboard=kb_list)
    return keyboard

def second_tap_keyboard(board: chess.Board, first_square_name: str) -> InlineKeyboardMarkup:
    square = chess.parse_square(first_square_name)
    piece = board.piece_at(square)

    kb_list = []
    for move in board.legal_moves:
        if move.from_square == square:
            to_square = chess.square_name(move.to_square)
            if move.promotion is not None:
                promotion = chess.piece_symbol(move.promotion)
                promotion_str = " → " + chess.Piece(move.promotion, chess.WHITE).unicode_symbol()
            else:
                promotion = promotion_str = ""

            kb_list.append([
                InlineKeyboardButton(
                    text=f"{piece.unicode_symbol()} → {to_square}{promotion_str}",
                    callback_data="turn2_" + to_square + promotion,
                )
            ])

    kb_list.append([
        InlineKeyboardButton(
            text=f"Отмена", callback_data="turn2_cancel",
        )
    ])
    keyboard = InlineKeyboardMarkup(inline_keyboard=kb_list)
    return keyboard


def make_move(board: chess.Board, move: str) -> Optional[chess.Outcome]:
    uci_move = chess.Move.from_uci(move)
    if not board.is_legal(uci_move):
        raise chess.IllegalMoveError
    board.push(uci_move)
    return board.outcome()


def make_bot_move(board: chess.Board) -> Optional[chess.Outcome]:
    bot_move = random.choice(list(board.legal_moves))
    board.push(bot_move)
    return board.outcome()
