import asyncio
import logging
import sys

from aiogram import Bot, Dispatcher, F
from aiogram.client.default import DefaultBotProperties
from aiogram.enums import ParseMode
from aiogram.filters import CommandStart, Command, StateFilter
from aiogram.types import Message, BufferedInputFile, CallbackQuery
from aiogram.fsm.context import FSMContext
from aiogram.fsm.state import StatesGroup, State

from my_credentials import API_TOKEN
from chess_utils import generate_board_image, first_tap_keyboard, second_tap_keyboard, make_move, make_bot_move
from board import UserBoard


dp = Dispatcher()


class ChessMatchStatesGroup(StatesGroup):
    has_chosen_piece = State()


@dp.message(CommandStart())
async def send_welcome(message: Message) -> None:
    await message.reply("Привет! Я ваш первый Telegram-бот.")

@dp.message(Command("newgame"))
async def start_game(message: Message) -> None:
    user_board = UserBoard(message.from_user.id)
    initial_board = await user_board.get_or_create()

    await message.answer_photo(
        photo=BufferedInputFile(generate_board_image(initial_board), "game.png"),
        caption="Игра началась! Делай ход. Нажми на клавиатуру: первым нажатием выбери фигуру, вторым - куда она пойдет",
        reply_markup=first_tap_keyboard(initial_board),
    )

@dp.callback_query(StateFilter(None), F.data.startswith("turn_"))
async def process_from_move(message: CallbackQuery, state: FSMContext) -> None:
    user_board = UserBoard(message.from_user.id)
    board = await user_board.get()
    if board is None:
        return

    square_name = message.data.removeprefix("turn_")
    await state.set_data({"square": square_name})
    await state.set_state(ChessMatchStatesGroup.has_chosen_piece)
    await message.message.edit_reply_markup(
        reply_markup=second_tap_keyboard(board, square_name),
    )


@dp.callback_query(ChessMatchStatesGroup.has_chosen_piece, F.data == "turn2_cancel")
async def process_to_cancel(message: Message, state: FSMContext) -> None:
    user_board = UserBoard(message.from_user.id)
    board = await user_board.get()
    if board is None:
        return

    await state.clear()
    await message.message.edit_reply_markup(
        reply_markup=first_tap_keyboard(board),
    )


@dp.callback_query(ChessMatchStatesGroup.has_chosen_piece, F.data.startswith("turn2_"))
async def process_to_move(message: Message, state: FSMContext) -> None:
    user_board = UserBoard(message.from_user.id)
    board = await user_board.get()
    if board is None:
        return

    async def send_end_of_game():
        await message.message.answer_photo(
            photo=BufferedInputFile(generate_board_image(board), "game.png"),
            caption=f"Игра закончена! Результат - {outcome.result()}. Новая игра - /newgame"
        )
        await message.message.delete()
        await state.clear()
        await user_board.delete()

    square_name = message.data.removeprefix("turn2_")
    data = await state.get_data()

    outcome = make_move(board, f"{data.get('square')}{square_name}")
    if outcome is not None:
        await send_end_of_game()
        return

    outcome = make_bot_move(board)
    if outcome is not None:
        await send_end_of_game()
        return

    await user_board.update(board)
    await message.message.answer_photo(
        photo=BufferedInputFile(generate_board_image(board), "game.png"),
        caption=f"Делайте следующий ход!",
        reply_markup=first_tap_keyboard(board),
    )
    await message.message.delete()
    await state.clear()


async def main() -> None:
    bot = Bot(token=API_TOKEN, default=DefaultBotProperties(parse_mode=ParseMode.HTML))
    await dp.start_polling(bot)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO, stream=sys.stdout)
    asyncio.run(main())
